using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Mvc;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Microsoft.IdentityModel.Tokens;
using ExperienceEfCore.Models;
using ExperienceEfCore.Data;
using ExperienceEfCore.Services;
using Microsoft.AspNetCore.Authorization;


namespace ExperienceEfCore.Controllers;

[Authorize]
[ApiController]
[Route("[controller]")]
public class AccountController : ControllerBase
{
    private MyDbContext _context;
    private readonly IConfiguration _configuration;
    private readonly UserManager<ApiUser> _userManager;
    private readonly SignInManager<ApiUser> _signManager;

    public AccountController(MyDbContext context, IConfiguration configuration, UserManager<ApiUser> userManager, SignInManager<ApiUser> signInManager){
        _context = context;
        _configuration = configuration;
        _userManager = userManager;
        _signManager = signInManager;
    }

    [AllowAnonymous]
    [HttpPost]
    [Route("Register")]
    public async Task<ActionResult> Register(RegisterDTO input)
    {
        try
        {
            if (ModelState.IsValid)
            {
                var success = await AccountServices.RegisterAsync(input, _userManager);
                return Ok(new { message = "Account registered", success });
            }
            else
            {
                var details = new ValidationProblemDetails(ModelState);
                details.Type =
                "https:/ /tools.ietf.org/html/rfc7231#section-6.5.1";
                details.Status = StatusCodes.Status400BadRequest;
                return new BadRequestObjectResult(details);
            }
        }
        catch (Exception e)
        {
            var exceptionDetails = new ProblemDetails();
            exceptionDetails.Detail = e.Message;
            exceptionDetails.Status =
            StatusCodes.Status500InternalServerError;
            exceptionDetails.Type =
            "https:/ /tools.ietf.org/html/rfc7231#section-6.6.1";
            return StatusCode(
            StatusCodes.Status500InternalServerError,
            exceptionDetails);
        }
    }

    [AllowAnonymous]
    [HttpPost]
    [Route("Login")] 
       public async Task<ActionResult> Login(LoginDTO input)
    {
        try
        {
            if (ModelState.IsValid)
            {
                var token = await AccountServices.LoginAsync(input, _userManager, _configuration);
                return Ok(token);
            }
            else
            {
                var details = new ValidationProblemDetails(ModelState);
                details.Type =
                "https:/ /tools.ietf.org/html/rfc7231#section-6.5.1";
                details.Status = StatusCodes.Status400BadRequest;
                return new BadRequestObjectResult(details);
            }
        }
        catch (Exception e)
        {
            var exceptionDetails = new ProblemDetails();
            exceptionDetails.Detail = e.Message;
            exceptionDetails.Status =
            StatusCodes.Status401Unauthorized;
            exceptionDetails.Type =
            "https:/ /tools.ietf.org/html/rfc7231#section-6.6.1";
            return StatusCode(
                StatusCodes.Status401Unauthorized, exceptionDetails);
        }
    }

    [AllowAnonymous]
    [HttpPost]
    [Route("SeedOneOfEveryRoleManager")] 
    public async Task<ActionResult> SeedOneOfEveryRoleAsync([FromServices] RoleManager<IdentityRole> roleManager)
    {
        try {
            // Seed roles
            await AccountServices.SeedRolesAsync(roleManager);

            await AccountServices.SeedUsersAsync(_userManager);

            return Ok(new { message = "All roles and accounts are seeded" });
        }
        catch (Exception e) {
            var exceptionDetails = new ProblemDetails();
            exceptionDetails.Detail = e.Message;
            exceptionDetails.Status =
            StatusCodes.Status500InternalServerError;
            exceptionDetails.Type =
            "https:/ /tools.ietf.org/html/rfc7231#section-6.6.1";
            return StatusCode(
                StatusCodes.Status500InternalServerError, exceptionDetails);
        }
    }
}