using ExperienceEfCore;
using ExperienceEfCore.Data;
using ExperienceEfCore.Models;
using ExperienceEfCore.Services;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Authorization;

[Authorize]
[ApiController]
[Route("[controller]")]
public class ExperienceController : ControllerBase
{
    private MyDbContext _context;

    public ExperienceController(MyDbContext context)
    {
        _context = context;
    }

    [AllowAnonymous]
    [HttpGet]
    public ActionResult<List<Experience>> GetAll() => ExperienceServices.GetAll(_context);

    [Authorize(Policy = "AdminOnly")]
    [HttpDelete("{id}")]
    public ActionResult DeleteExperience(int id)
    {
        var experience = ExperienceServices.DeleteExperience(_context, id);
        if (experience == null)
        {
            return NotFound(); // 404 if not found
        }

        return NoContent(); // 204 No Content after successful delete
    }
    
    [Authorize(Policy = "AdminOnly")]
    [HttpPut("UpdateExperiencePrice")]
    public ActionResult UpdateExperiencePrice([FromBody] UpdateExperiencePriceModelDTO model)
    {
        if (!ModelState.IsValid)
        {
            return BadRequest(ModelState); // 400 Bad Request if model state is invalid
        }
        var updatedExperience = ExperienceServices.UpdateExperiencePrice(_context, model.Id, model.Price);

        if (updatedExperience == null)
        {
            return NotFound(); // 404 if not found
        }

        return NoContent(); // 204 No Content after successful update
    }
    
    [Authorize(Policy = "AdminOnly")]
    [HttpPost("AddExperience")]
    public ActionResult<Experience> AddExperience(int providerId, string name, string description, int price)
    {
        var experience = new Experience
        {
            Provider = ProviderService.GetProvider(_context, providerId),
            Name = name,
            Description = description,
            Price = price
        };

        ExperienceServices.AddExperience(_context, experience);
        return NoContent();
    }

    [AllowAnonymous]
    [HttpGet("ExperienceNameBySharedExperience")]
    public ActionResult<List<string>> GetExperiencesBySharedExperienceId(int sharedExperienceId)
    {
        var experiences = ExperienceServices.GetExperiencesBySharedExperienceId(_context, sharedExperienceId);
        if (experiences == null || experiences.Count == 0)
        {
            return NotFound("No experiences found for the given shared experience ID.");
        }
        return Ok(experiences); // 200 OK with the list of experiences
    }

    [Authorize(Policy = "ManagerOnly")]
    [HttpGet("GetExperienceStatistics")]
    public ActionResult<List<ExpInfoDTO>> GetExperienceStatistics()
    {
        var statistics = ExperienceServices.GetExperienceStatistics(_context);
        if (statistics == null || statistics.Count == 0)
        {
            return NotFound("No experience statistics available.");
        }
        return Ok(statistics); // 200 OK with the experience statistics
    }

    [Authorize(Policy = "AdminOnly")]
    [HttpGet("GetExperienceSalesWithDiscount")]
    public ActionResult<List<ExpInfoDiscountDTO>> GetExperienceSalesWithDiscount()
    {
        var salesWithDiscount = ExperienceServices.GetExperienceSalesWithDiscount(_context);
        if (salesWithDiscount == null || salesWithDiscount.Count == 0)
        {
            return NotFound("No experience sales data with discounts available.");
        }
        return Ok(salesWithDiscount); // 200 OK with the sales data
    }

    [Authorize(Policy = "ManagerOnly")]
    [HttpGet("GetPriceMinAvgMax")]
    public ActionResult<PriceMinMaxAvgDTO> GetPriceStatsAsync() 
    {
        var priceStats = ExperienceServices.GetPriceStats(_context);
        if (priceStats == null)
        {
            return NotFound(); // 404 if not found
        }
        return Ok(priceStats);
    }

}
