using ExperienceEfCore.Attributes;
using Microsoft.AspNetCore.Mvc;
using ExperienceEfCore.Models;
using ExperienceEfCore.Services;
using ExperienceEfCore.Data;
using Microsoft.AspNetCore.Identity;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using Microsoft.IdentityModel.Tokens;

namespace ExperienceEfCore.Services
{
    public static class AccountServices
    {
        static AccountServices()
        {
        }   

        public static async Task<bool> RegisterAsync(RegisterDTO input, UserManager<ApiUser> _userManager)
        {
            var newUser = new ApiUser();

                newUser.UserName = input.Email;
                newUser.Email = input.Email;
                newUser.FullName = input.FullName;
                
                var result = await _userManager.CreateAsync(newUser, input.Password);

                if (result.Succeeded)
                {
                    return true;
                }
                else {
                    throw new Exception(
                    string.Format("Error: {0}", string.Join(" ",
                    result.Errors.Select(e => e.Description))));
                }
        }

        public static async Task<string> LoginAsync(LoginDTO input, UserManager<ApiUser> _userManager, IConfiguration _configuration)
        {
            var user = await _userManager.FindByNameAsync(input.UserName);
            if (user == null || !await _userManager.CheckPasswordAsync(user, input.Password))
                throw new Exception("Invalid login attempt.");
            else
            {
                var signingCredentials = new SigningCredentials(
                        new SymmetricSecurityKey(
                        System.Text.Encoding.UTF8.GetBytes(_configuration["JWT:SigningKey"])),
                        SecurityAlgorithms.HmacSha256);

                var claims = new List<Claim>();
                claims.Add(new Claim(ClaimTypes.Name, user.UserName));
                var userClaims = await _userManager.GetClaimsAsync(user);

                var isAdminClaim = userClaims.FirstOrDefault(c => c.Type == "IsAdmin" && c.Value == "true");
                var isManagerClaim = userClaims.FirstOrDefault(c => c.Type == "IsManager" && c.Value == "true");
                var isProviderClaim = userClaims.FirstOrDefault(c => c.Type == "IsProvider" && c.Value == "true");
                var isGuestClaim = userClaims.FirstOrDefault(c => c.Type == "IsGuest" && c.Value == "true");

                if (isAdminClaim != null)
                {
                    claims.Add(new Claim("IsAdmin", "true"));
                }
                else if (isManagerClaim != null)
                {
                    claims.Add(new Claim("IsManager", "true"));
                }
                else if (isProviderClaim != null)
                {
                    claims.Add(new Claim("IsProvider", "true"));
                }
                else if (isGuestClaim != null)
                {
                    claims.Add(new Claim("IsGuest", "true"));
                }

                // Ensure claims are serialized into the JWT
                var jwtObject = new JwtSecurityToken(
                        issuer: _configuration["JWT:Issuer"],
                        audience: _configuration["JWT:Audience"],
                        claims: claims,
                        expires: DateTime.Now.AddSeconds(300),
                        signingCredentials: signingCredentials);

                var jwtString = new JwtSecurityTokenHandler()
                .WriteToken(jwtObject);
                
                return jwtString;
            }
        }

        [HttpPut("SeedUsers")]
        public static async Task<IActionResult> SeedUsersAsync(UserManager<ApiUser> _userManager)
        {
            // Admin user
            var adminUser = new ApiUser
            { 
                UserName = "Admin",
                Email = "Admin@localhost",
                FullName = "Admin" 
            };

            if (await _userManager.FindByNameAsync(adminUser.UserName) == null)
            {
                var result = await _userManager.CreateAsync(adminUser, "StrongPassword123!");
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create admin user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Admin" role to the user
                    var roleAdded = await _userManager.AddClaimAsync(adminUser, new Claim("IsAdmin", "true"));
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Admin' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }

            // Manager user
            var managerUser = new ApiUser
            { 
                UserName = "Manager",
                Email = "Manager@localhost",
                FullName = "Manager" 
            };

            if (await _userManager.FindByNameAsync(managerUser.UserName) == null)
            {
                var result = await _userManager.CreateAsync(managerUser, "StrongPassword123!");
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create manager user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Manager" role to the user
                    var roleAdded = await _userManager.AddClaimAsync(managerUser, new Claim("IsManager", "true"));
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Manager' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }

            // Provider user
            var providerUser = new ApiUser
            { 
                UserName = "Provider",
                Email = "Provider@localhost",
                FullName = "Provider" 
            };

            if (await _userManager.FindByNameAsync(providerUser.UserName) == null)
            {
                var result = await _userManager.CreateAsync(providerUser, "StrongPassword123!");
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create provider user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Provider" role to the user
                    var roleAdded = await _userManager.AddClaimAsync(providerUser, new Claim("IsProvider", "true"));
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Provider' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }

            // Guest user
            var guestUser = new ApiUser
            { 
                UserName = "Guest",
                Email = "Guest@localhost",
                FullName = "Guest" 
            };

            if (await _userManager.FindByNameAsync(guestUser.UserName) == null)
            {
                var result = await _userManager.CreateAsync(guestUser, "StrongPassword123!");
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create guest user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Guest" role to the user
                    var roleAdded = await _userManager.AddClaimAsync(guestUser, new Claim("IsGuest", "true"));
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Guest' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }

            return new OkResult(); // Ensure a value is returned
        } 

        public static async Task SeedAdminAsync(UserManager<ApiUser> userManager)
        {
            const string adminEmail = "Admin@localhost";
            const string adminPassword = "StrongPassword123!";
            if (userManager == null)
            {
            throw new ArgumentNullException(nameof(userManager));
            }
            if (await userManager.FindByNameAsync(adminEmail) == null)
            {
            var adminUser = new ApiUser { UserName = adminEmail, Email = adminEmail, FullName = "Admin" };

            var result = await userManager.CreateAsync(adminUser, adminPassword);
            if (!result.Succeeded)
            {
                throw new Exception($"Failed to create admin user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
            }
            else
            {
                // Assign the "Admin" role to the user
                var roleAdded = await userManager.AddToRoleAsync(adminUser, "Admin");
                if (!roleAdded.Succeeded)
                {
                throw new Exception($"Failed to assign 'Admin' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                }
            }
            }
        }

        public static async Task SeedManagerAsync(UserManager<ApiUser> userManager)
        {
            const string managerEmail = "Manager@localhost";
            const string managerPassword = "StrongPassword123!";
            if (userManager == null)
            {
                throw new ArgumentNullException(nameof(userManager));
            }
            if (await userManager.FindByNameAsync(managerEmail) == null)
            {
                var managerUser = new ApiUser { UserName = managerEmail, Email = managerEmail, FullName = "Manager" };

                var result = await userManager.CreateAsync(managerUser, managerPassword);
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create manager user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Manager" role to the user
                    var roleAdded = await userManager.AddToRoleAsync(managerUser, "Manager");
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Manager' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }
        }

        public static async Task SeedProviderAsync(UserManager<ApiUser> userManager)
        {
            const string providerEmail = "Provider@localhost";
            const string providerPassword = "StrongPassword123!";
            if (userManager == null)
            {
                throw new ArgumentNullException(nameof(userManager));
            }
            if (await userManager.FindByNameAsync(providerEmail) == null)
            {
                var providerUser = new ApiUser { UserName = providerEmail, Email = providerEmail, FullName = "Provider" };

                var result = await userManager.CreateAsync(providerUser, providerPassword);
                if (!result.Succeeded)
                {
                    throw new Exception($"Failed to create provider user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Provider" role to the user
                    var roleAdded = await userManager.AddToRoleAsync(providerUser, "Provider");
                    if (!roleAdded.Succeeded)
                    {
                        throw new Exception($"Failed to assign 'Provider' role: {string.Join(", ", roleAdded.Errors.Select(e => e.Description))}");
                    }
                }
            }
        }

        public static async Task SeedGuestAsync(UserManager<ApiUser> userManager)
        {
            const string guestEmail = "Guest@localhost";
            const string guestPassword = "StrongPassword123!";
            if (userManager == null)
            {
                throw new ArgumentNullException(nameof(userManager));
            }
            if (await userManager.FindByNameAsync(guestEmail) == null)
            {
                var guestUser = new ApiUser { UserName = guestEmail, Email = guestEmail, FullName = "Guest" };

                var result = await userManager.CreateAsync(guestUser, guestPassword);
                if (!result.Succeeded)
                {
                    //throw new Exception($"Failed to create guest user: {string.Join(", ", result.Errors.Select(e => e.Description))}");
                }
                else
                {
                    // Assign the "Guest" role to the user
                    var roleAdded = await userManager.AddToRoleAsync(guestUser, "Guest");
                    if (!roleAdded.Succeeded)
                    {
                        //
                    }
                }
            }
        }

        public static async Task SeedRolesAsync(RoleManager<IdentityRole> roleManager)
        {
            var roles = new[] { "Admin", "Manager", "Provider", "Guest" };

            foreach (var role in roles)
            {
                if (!await roleManager.RoleExistsAsync(role))
                {
                    var result = await roleManager.CreateAsync(new IdentityRole(role));
                    if (!result.Succeeded)
                    {
                        throw new Exception($"Failed to create role '{role}': {string.Join(", ", result.Errors.Select(e => e.Description))}");
                    }
                }
            }
        }
    } 
}