﻿using ExperienceEfCore.Data;
using ExperienceEfCore.Models;
using Microsoft.AspNetCore.Authorization.Infrastructure;
using Microsoft.EntityFrameworkCore;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;

namespace ExperienceEfCore.Services
{
    public static class ExperienceServices 
    {

        static ExperienceServices(){

        }
        public static List<Experience> GetAll(MyDbContext context)
        {
            return context.Experiences.ToList();
        }

       internal static void AddExperience(MyDbContext context, Experience experience)
        {
            context.Experiences.Add(experience);
            context.SaveChanges();
        }


        internal static Experience? DeleteExperience(MyDbContext context, int id)
        {
            var experience = context.Experiences.Find(id);
            if (experience == null)
            {
                return null;
            }

            context.Experiences.Remove(experience);
            context.SaveChanges();
            return experience;
        }

        internal static Experience? UpdateExperiencePrice(MyDbContext context, int id, int newPrice)
        {
            var existingExperience = context.Experiences.Find(id);
            if (existingExperience == null)
            {
                return null;
            }

            // Update only the Price field
            existingExperience.Price = newPrice;

            context.SaveChanges();
            return existingExperience;
        }


        public static List<PriceMinMaxAvgDTO> GetPriceStats(MyDbContext context)
        {
            // Calculate the min, max, and average price of all experiences
            var priceStats = context.Experiences
                .Where(e => e.Price.HasValue) // Ensure we only consider experiences with a valid price (nullable check)
                .Select(e => new PriceMinMaxAvgDTO
                {
                    MinPrice = context.Experiences.Min(ex => ex.Price ?? 0),  // Get the minimum price (handling nulls)
                    MaxPrice = context.Experiences.Max(ex => ex.Price ?? 0),  // Get the maximum price (handling nulls)
                    AvgPrice = context.Experiences.Average(ex => ex.Price ?? 0) // Get the average price (handling nulls)
                })
                .Distinct() // Ensure a single result is returned since Min, Max, and Avg are the same for the entire list of experiences
                .ToList();

            return priceStats;
        }
        public static List<string> GetExperiencesBySharedExperienceId(MyDbContext context, int sharedExperienceId)
        {
            var ExperienceTitles = context.Experiences
           .Where(e => e.SharedExperiences.Any(se => se.Id == sharedExperienceId))
           .Select(g => g.Name)
           .Distinct()
           .ToList();

            return ExperienceTitles;
        }

        // Get the experiences, number of guests and total price for the experience
        public static List<ExpInfoDTO> GetExperienceStatistics(MyDbContext context)
        {
            var result = context.Experiences
                .Select(e => new ExpInfoDTO
                {
                    NumberOfGuests = e.Guests.Count(),
                    ExperienceName = e.Name,
                    TotalSales = e.Guests.Count() * (e.Price ?? 0) // Handle null Price
                })
                .ToList();

            return result;
        }
        public static List<ExpInfoDiscountDTO> GetExperienceSalesWithDiscount(MyDbContext context)
        {
            var result = context.Experiences
                .Select(e => new ExpInfoDiscountDTO
                {
                    ExperienceName = e.Name,
                    NumberOfGuests = e.Guests.Count(),
                    TotalSales = e.Guests.Count() * (e.Price ?? 0), // Handle nullable Price
                    TotalSalesDiscount = e.Guests.Count() >= 4
                        ? e.Guests.Count() * (e.Price ?? 0) * 0.80  // 20% discount
                        : e.Guests.Count() >= 2
                            ? e.Guests.Count() * (e.Price ?? 0) * 0.90  // 10% discount
                            : e.Guests.Count() * (e.Price ?? 0)  // No discount
                })
                .OrderBy(x => x.ExperienceName)
                .ToList();

            return result;
        }

    }
}
